unit ID3TagTools;

interface

uses
  SysUtils, Types;

type
  sarray = array[0..3] of byte;
  psarray = ^sarray;

  ID3v2Header = packed record
     identifier: array[0..2] of char;
     major_version: byte;
     minor_version: byte;
     flags: byte;
     size: DWord;
  end;


implementation

function size_decodeh(insize: DWord): DWord;
 { decodes the size headers only, which does not use bit 7 in each byte,
   requires MSB conversion as well }
 var
   outdval: DWord;
   outd, ind: psarray;
   tnext2, pnext2: byte;

 begin
   outdval := 0;
   outd := @outdval;
   ind := @insize;
   tnext2 := ind^[2] shr 1;
   pnext2 := ind^[1] shr 2;

   outd^[0] := ind^[3] or ((ind^[2] and $01) shl 7);
   outd^[1] := tnext2 or ((ind^[1] and $03) shl 6);
   outd^[2] := pnext2 or ((ind^[0] and $07) shl 5);
   outd^[3] := ind^[0] shr 3;
   Result := outdval;
 end;

procedure ID3v2_LoadData(filename: string; var memrec: pointer;
                       var outsize: integer);
  { procedure loads ID3v2 data from "filename".  Returns outsize = 0 if
    there is no ID3v2 data }

var
  infile: file;
  v1h: ID3V2Header;
begin
  assign(infile, filename);
  reset(infile, 1);
// read main header to get id3v2 size
  blockread(infile, v1h, sizeof(v1h));
// detect if there is id3v2 data
  if v1h.identifier = 'ID3' then
    begin
      outsize := size_decodeh(v1h.size);
      // read ID3v2 header data
      getmem(memrec, outsize);
      blockread(infile, memrec^, outsize);
      Close(infile);
    end
  else
    outsize := 0;
  end;

function id3v2_erase(infilestr: string): boolean;
{ erase all ID3v2 data.  Data are stored at the beginning of file, so file
  must be rewritten }
 const
   tempfilename = 'TMp@!0X.MP3';
 var
   memrec: pointer;
   outsize, dataread: integer;
   IsID3v2: boolean;
   databuffer: array[1..32768] of byte;
   newfile, origfile: file;
 begin
// reuse service routine to get information
   Id3v2_loaddata(infilestr, memrec, outsize);
// is there ID3v2 data?
   if outsize > 0 then
     begin
      // need to clean up after the service routine
       freemem(memrec);
      // get amount of data to erase
      outsize := outsize + sizeof(Id3v2Header);
      writeln('Data to delete is: ', outsize, ' bytes.');
      // now rewrite the file
      AssignFile(origfile, infilestr);
      reset(origfile, 1);
      AssignFile(newfile, tempfilename);
      rewrite(newfile, 1);
      Seek(origfile, outsize);
      repeat
        blockread(origfile, databuffer, sizeof(databuffer), dataread);
        blockwrite(newfile, databuffer, dataread);
      until dataread = 0;
      CloseFile(origfile);
      CloseFile(newfile);
     // rename temp file and delete original
      DeleteFile(infilestr);
      RenameFile(tempfilename, infilestr);
      IsID3v2 := true;
    end
  else
     IsID3v2 := false;
  Result := IsID3v2;
end;

end.
